@@ -23,7 +23,6 @@ import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_serialization import base64
-from oslo_service import loopingcall
 from oslo_utils import imageutils
 import requests
 
@@ -316,7 +315,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
 
     def _test_make_partitions(self, mock_exc, boot_option, boot_mode='bios',
                               disk_label=None):
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
                                    self.node_uuid, boot_option=boot_option,
@@ -387,7 +386,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'mkpart', 'primary', '', '2561', '3585']
         self.dev = 'fake-dev'
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
                                    self.node_uuid)
@@ -411,7 +410,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': swap,
                            'root': root}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -434,7 +433,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': swap,
                            'root': root}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -454,7 +453,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': 'fake-dev2',
                            'root': 'fake-dev3'}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -465,7 +464,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         self.assertEqual(expected_result, result)
 
 
-@mock.patch.object(utils, 'execute', autospec=True)
+@mock.patch.object(utils, 'execute', autospec=True, return_value=('', ''))
 class DestroyMetaDataTestCase(base.IronicLibTestCase):
 
     def setUp(self):
@@ -479,7 +478,10 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
                                     use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
-                                    use_standard_locale=True)]
+                                    use_standard_locale=True),
+                          mock.call('fuser', self.dev,
+                                    check_exit_code=[0, 1],
+                                    run_as_root=True)]
         disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
         mock_exec.assert_has_calls(expected_calls)
 
@@ -514,7 +516,8 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         mock_exec.side_effect = iter(
             [processutils.ProcessExecutionError(description='--force'),
              (None, None),
-             (None, None)])
+             (None, None),
+             ('', '')])
 
         expected_call = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                    run_as_root=True,
@@ -562,13 +565,7 @@ class PopulateImageTestCase(base.IronicLibTestCase):
         self.assertFalse(mock_dd.called)
 
 
-def _looping_call_done(*args, **kwargs):
-    raise loopingcall.LoopingCallDone()
-
-
-@mock.patch.object(disk_partitioner.DiskPartitioner,
-                   '_wait_for_disk_to_become_available',
-                   _looping_call_done)
+@mock.patch.object(utils, 'wait_for_disk_to_become_available', lambda *_: None)
 @mock.patch.object(disk_utils, 'is_block_device', lambda d: True)
 @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
 @mock.patch.object(disk_utils, 'dd', lambda *_: None)
