@@ -198,7 +198,7 @@ class ParseRootDeviceTestCase(base.IronicLibTestCase):
             'serial': 'foo-serial', 'vendor': 'foo VENDOR with space',
             'name': '/dev/sda', 'wwn_with_extension': '123456111',
             'wwn_vendor_extension': '111', 'rotational': True,
-            'hctl': '1:0:0:0'}
+            'hctl': '1:0:0:0', 'by_path': '/dev/disk/by-path/1:0:0:0'}
         result = utils.parse_root_device_hints(root_device)
         expected = {
             'wwn': 's== 123456', 'model': 's== foo%20model',
@@ -206,7 +206,8 @@ class ParseRootDeviceTestCase(base.IronicLibTestCase):
             'vendor': 's== foo%20vendor%20with%20space',
             'name': 's== /dev/sda', 'wwn_with_extension': 's== 123456111',
             'wwn_vendor_extension': 's== 111', 'rotational': True,
-            'hctl': 's== 1%3A0%3A0%3A0'}
+            'hctl': 's== 1%3A0%3A0%3A0',
+            'by_path': 's== /dev/disk/by-path/1%3A0%3A0%3A0'}
         self.assertEqual(expected, result)
 
     def test_parse_root_device_hints_with_operators(self):
@@ -216,13 +217,14 @@ class ParseRootDeviceTestCase(base.IronicLibTestCase):
             'name': '<or> /dev/sda <or> /dev/sdb',
             'wwn_with_extension': 's!= 123456111',
             'wwn_vendor_extension': 's== 111', 'rotational': True,
-            'hctl': 's== 1:0:0:0'}
+            'hctl': 's== 1:0:0:0', 'by_path': 's== /dev/disk/by-path/1:0:0:0'}
 
         # Validate strings being normalized
         expected = copy.deepcopy(root_device)
         expected['model'] = 's== foo%20model'
         expected['vendor'] = 's== foo%20vendor%20with%20space'
         expected['hctl'] = 's== 1%3A0%3A0%3A0'
+        expected['by_path'] = 's== /dev/disk/by-path/1%3A0%3A0%3A0'
 
         result = utils.parse_root_device_hints(root_device)
         # The hints already contain the operators, make sure we keep it
@@ -307,6 +309,10 @@ class ParseRootDeviceTestCase(base.IronicLibTestCase):
         self.assertRaises(ValueError, utils.parse_root_device_hints,
                           {'hctl': 123})
 
+    def test_parse_root_device_hints_invalid_by_path(self):
+        self.assertRaises(ValueError, utils.parse_root_device_hints,
+                          {'by_path': 123})
+
     def test_parse_root_device_hints_non_existent_hint(self):
         self.assertRaises(ValueError, utils.parse_root_device_hints,
                           {'non-existent': 'foo'})
@@ -457,3 +463,150 @@ class MatchRootDeviceTestCase(base.IronicLibTestCase):
         dev = utils.match_root_device_hints(empty_dev, {'model': 'foo'})
         self.assertIsNone(dev)
         self.assertTrue(mock_warn.called)
+
+
+class WaitForDisk(base.IronicLibTestCase):
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available(self, mock_exc):
+        mock_exc.return_value = ('', '')
+        utils.wait_for_disk_to_become_available('fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(1, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True,
+                       side_effect=processutils.ProcessExecutionError(
+                           stderr='fake'))
+    def test_wait_for_disk_to_become_available_no_fuser(self, mock_exc):
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+        self.assertRaises(exception.IronicException,
+                          utils.wait_for_disk_to_become_available,
+                          'fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available_device_in_use_psmisc(
+            self, mock_exc):
+        # Test that the device is not available. This version has the 'psmisc'
+        # version of 'fuser' values for stdout and stderr.
+        # NOTE(TheJulia): Looks like fuser returns the actual list of pids
+        # in the stdout output, where as all other text is returned in
+        # stderr.
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+        # The 'psmisc' version has a leading space character in stdout. The
+        # filename is output to stderr
+        mock_exc.side_effect = [(' 1234   ', 'fake-dev: '),
+                                (' 15503  3919 15510 15511', 'fake-dev:')]
+        expected_error = ('Processes with the following PIDs are '
+                          'holding device fake-dev: 15503, 3919, 15510, '
+                          '15511. Timed out waiting for completion.')
+        self.assertRaisesRegex(
+            exception.IronicException,
+            expected_error,
+            utils.wait_for_disk_to_become_available,
+            'fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available_device_in_use_busybox(
+            self, mock_exc):
+        # Test that the device is not available. This version has the 'busybox'
+        # version of 'fuser' values for stdout and stderr.
+        # NOTE(TheJulia): Looks like fuser returns the actual list of pids
+        # in the stdout output, where as all other text is returned in
+        # stderr.
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+        # The 'busybox' version does not have a leading space character in
+        # stdout. Also nothing is output to stderr.
+        mock_exc.side_effect = [('1234', ''),
+                                ('15503  3919 15510 15511', '')]
+        expected_error = ('Processes with the following PIDs are '
+                          'holding device fake-dev: 15503, 3919, 15510, '
+                          '15511. Timed out waiting for completion.')
+        self.assertRaisesRegex(
+            exception.IronicException,
+            expected_error,
+            utils.wait_for_disk_to_become_available,
+            'fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available_no_device(self, mock_exc):
+        # NOTE(TheJulia): Looks like fuser returns the actual list of pids
+        # in the stdout output, where as all other text is returned in
+        # stderr.
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+
+        mock_exc.return_value = ('', 'Specified filename /dev/fake '
+                                     'does not exist.')
+        expected_error = ('Fuser exited with "Specified filename '
+                          '/dev/fake does not exist." while checking '
+                          'locks for device fake-dev. Timed out waiting '
+                          'for completion.')
+        self.assertRaisesRegex(
+            exception.IronicException,
+            expected_error,
+            utils.wait_for_disk_to_become_available,
+            'fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available_dev_becomes_avail_psmisc(
+            self, mock_exc):
+        # Test that initially device is not available but then becomes
+        # available. This version has the 'psmisc' version of 'fuser' values
+        # for stdout and stderr.
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+        # The 'psmisc' version has a leading space character in stdout. The
+        # filename is output to stderr
+        mock_exc.side_effect = [(' 1234   ', 'fake-dev: '),
+                                ('', '')]
+        utils.wait_for_disk_to_become_available('fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_wait_for_disk_to_become_available_dev_becomes_avail_busybox(
+            self, mock_exc):
+        # Test that initially device is not available but then becomes
+        # available. This version has the 'busybox' version of 'fuser' values
+        # for stdout and stderr.
+        CONF.disk_partitioner.check_device_interval = .01
+        CONF.disk_partitioner.check_device_max_retries = 2
+        # The 'busybox' version does not have a leading space character in
+        # stdout. Also nothing is output to stderr.
+        mock_exc.side_effect = [('1234 5895', ''),
+                                ('', '')]
+        utils.wait_for_disk_to_become_available('fake-dev')
+        fuser_cmd = ['fuser', 'fake-dev']
+        fuser_call = mock.call(*fuser_cmd, run_as_root=True,
+                               check_exit_code=[0, 1])
+        self.assertEqual(2, mock_exc.call_count)
+        mock_exc.assert_has_calls([fuser_call, fuser_call])
